#--------------------------------------------------------------
#
#   NIR Image data reduciton procedure (for OASIS and CISCO)
#
#   #Step 0)  Dark Frame Subtraction
#   Step 1)  Flat Fielding Each Image
#   Step 2)  Background Level Correction
#   Step 3)  Image Combine ( Sub-Group )
#               using average with sigclip
#   Step 4)  Make Background Image
#               using median combine
#                  = CR, bad pixel rejection
#   #Step 5)  Background Image Subtraction
#   #Step 6)  Image Shift
#   #Step 7)  Seeing Correction
#   #Step 8)  Image Combine
#   #            using median combine
#   #               = CR, bad pixel rejection
#
#        98/01/23  by Masayuki Akiyama
#            shift_comb.cl -> irred.cl
#        98/02/20  by F. Iwamuro
#            irred.cl -> mksubgr.cl
#-------------------------------------------------------------
#    !! Need imstatrej.cl !!
#-------------------------------------------------------------

procedure	mksubgr( imlist, output, ffimage, n_image, bg_value, \
         xmin, xmax, ymin, ymax, bg_log, display )

file	imlist		{prompt = "Input image list with @ mark"}
file    output          {prompt = "Output image name"}
file    ffimage         {prompt = "Normarized Flat Field image (DomeON - DomeOFF) "}
int     n_image         {prompt = "Input number of images in each sub-group"}
real    bg_value        {prompt = "Nominal background value"}
int     xmin            {prompt = "Effective image area : xmin"}
int     xmax            {prompt = "Effective image area : xmax"}
int     ymin            {prompt = "Effective image area : ymin"}
int     ymax            {prompt = "Effective image area : ymax"}
file    bg_log          {prompt = "Log file of background value of each frame"}
int     display         {prompt = "1: Diaplay images; 0: Don't display images"}

struct  *list1    {mode = 'h'}
struct  *list2_2  {mode = 'h'}
#
#
begin
        string  tmpfile1, tmpfile2, tmpfile3, tmpfile4, tmpfile5
        string  tmpfile2_2, tmpfile2_3, tmpfile3_2, tmpfile3_1, tmpfile5_1

	string 	junk1, junk2
	real 	weight, weighttot

	string  filelist1

        string  tmpimage1, tmpimage3_2
        string  tmpimage1_1, tmpimage1_2, tmpimage1_3, tmpimage2_2

	string  output1, ffimage1

	string  bg_log1

        real    bg_value1
	real    bg_shift
 
        int     i, j, nimage
        int     x1, x2, y1, y2

 	int 	dispparam

	filelist1 = imlist
	tmpfile1 = mktemp( "tmp_file." )
	dispparam = display

# ---- Image Name or Image List with @ mark ?
	sections( filelist1, option='fullname', > tmpfile1 )
	list1 = tmpfile1

	output1 = output
	ffimage1 = ffimage
	 
        nimage = n_image

        x1 = xmin
        x2 = xmax
        y1 = ymin
        y2 = ymax

        bg_value1 = bg_value

	bg_log1 = bg_log

	junk1=""
	junk2=""

#        reset stdimage = imt1024
#        task imstatrej = imstatrej.cl

#########################################
#---- Step 1-3
#---- F.F., Background-correction and
#---- Make Combined Image (sub-group).
#########################################
	
	tmpfile2    = mktemp( "tmp_file." )
	tmpfile2_2  = mktemp( "tmp_file." )
#        tmpfile2_3  = mktemp( "tmp_file." )
	tmpfile3    = mktemp( "tmp_file." )
        tmpfile3_1  = mktemp( "tmp_file." )
        tmpfile3_2  = mktemp( "tmp_file." )
	tmpfile4    = mktemp( "tmp_file." )
	tmpfile5    = mktemp( "tmp_file." )
	tmpfile5_1    = mktemp( "tmp_file." )

	i=0
        j=0
        weighttot=0
	while( fscanf( list1, "%f %s",weight, junk1) != EOF) 
        {
                print(junk1)
# Step 1 and 2 :  Flat Fielding and Background Correction
                tmpimage1_1 = mktemp( "tmp_im." )
                tmpimage1_2 = mktemp( "tmp_im." )
                tmpimage1_3 = mktemp( "tmp_im." )

#                imarith( junk1, "/", ffimage1, tmpimage1_2 )
                imcopy( junk1, tmpimage1_2 )
   
                imstatrej( tmpimage1_2//"["//x1//":"//x2//","//y1//":"//y2//"]" , 3, >> tmpfile2_2 )
    
                list2_2 = tmpfile2_2
         	while( fscan( list2_2, junk2, bg_shift ) == 2 )
                {
                      bg_shift = bg_shift - bg_value1
                      print( bg_shift )
                }
                imarith( tmpimage1_2,"-",bg_shift, tmpimage1_3 )
          
                imdelete( tmpimage1_2 )
                delete( tmpfile2_2, ver-, >&"dev$null" )

                imarith( tmpimage1_3, "/", ffimage1, tmpimage1_3 )
                # overwrite flat-field image
                imdelete(junk1)
                imrename(tmpimage1_3,junk1)

                if(weight != 0){
                  print( junk1, >> tmpfile2 )
                  print( weight, >> tmpfile4 )
                }
                weighttot += weight

                print( junk1, "    ", bg_value1, bg_shift, >> bg_log1 )

# Step 3 : Image Combine
 
		i = i+1
		if( i == nimage )
		   {
		   tmpimage1 = mktemp( "tmp_im.")
	           print( tmpimage1, >> tmpfile3 )
                   if(weighttot == 0) imcopy(tmpimage1_3, tmpimage1)
                   else{
                     imcomb("@"//tmpfile2 ,tmpimage1,combine='average',reject='sigclip',outtype='real',lthresh=INDEF,hthresh=INDEF,lsigma=2, hsigma=2, weight="@"//tmpfile4)
#                     imdelete( "@"//tmpfile2_3 )
#                     imrename( "@"//tmpfile2 , "@"//tmpfile2_3 )
                     delete( tmpfile2,ver-,>&"dev$null" )
                   }

                   print( "overwriting..." )
                   delete( tmpfile4,ver-,>&"dev$null" )
#                   delete( tmpfile2_3,ver-,>&"dev$null" )
		   i = 0
                   j = j + 1
                   print( "Frame", j, " ended" )
                   if(j<10) print( output1,".00",j, >> tmpfile3_2 )
                   else if(j<100) print( output1,".0",j, >> tmpfile3_2 )
                   else print( output1,".",j, >> tmpfile3_2 )
                   print(weighttot, >> tmpfile5)
                   print("weighttot= ",weighttot)
                   weighttot=0 
	           }
	}

	print('F.F. and Background Correction were Ended') 
        print('Image Combination in each sub-group was Ended')

#########################################
#---- Step 4
#---- Make Background Frame
#########################################
	
        imrename( "@"//tmpfile3,"@"//tmpfile3_2 )
        list1 = tmpfile3_2
        list2_2 = tmpfile5
	while( fscan( list1, junk1) != EOF) 
        {
          if( fscan( list2_2, weight ) != EOF)
            if( weight !=0 ){
              print(junk1, >> tmpfile3_1)
              print(weight, >> tmpfile5_1)
            }
        }
	imcomb("@"//tmpfile3_1 ,output1,combine='median',reject='sigclip',outtype='real',lthresh=INDEF,hthresh=INDEF,lsigma=2, hsigma=2,  weight="@"//tmpfile5_1)
        if( dispparam == 1 ){		
          display(output1,1,fi+,zs+,zr+)
        }

finish:
        delete( tmpfile1,ver-,>&"dev$null" )
        delete( tmpfile3,ver-,>&"dev$null" )
        delete( tmpfile3_1,ver-,>&"dev$null" )
        delete( tmpfile3_2,ver-,>&"dev$null" )
        delete( tmpfile5,ver-,>&"dev$null" )
        delete( tmpfile5_1,ver-,>&"dev$null" )
        print( 'This procedure ended.' )
        bye
end
