#!/bin/sh

if [ $# -ne 0 ]
then
    echo "Usage : "$0
    exit 1
fi

## Set Debug Parameter 
# DEBUG=1; erase all temporary files 
#       0; leave all temporary files 
DEBUG=1 

TMPFILELIST=tmpfile.list
GLOBALTMPFILELIST=globaltmpfile.list
GLOBALTMPIMGLIST=globaltmpimg.list

CLLOG=log/procclean.`date +%y%m%d%H%M%S`.log 
CLCOM=cl.com 
TMPLOG=tmp.log

if [ -f $TMPFILELIST ]
then
  \rm $TMPFILELIST
fi

if [ ! -d log ]
then 
  mkdir log 
fi
if [ -f $CLLOG ]
then 
  rm $CLLOG 
fi
 
echo "IRAF Log of $0 $1 $2 $3" > $CLLOG 

echo $CLCOM >> $TMPFILELIST 
echo $TMPLOG >> $TMPFILELIST 
 
echo "### Erase Temporary Frames ###"
echo "### Erase Temporary Frames ###" >> $CLLOG
if [ -f $GLOBALTMPIMGLIST ]; then
  sed 's/^/imdel /g' $GLOBALTMPIMGLIST > $CLCOM
fi
if [ -f ldata.tbl ]; then
  echo "imdel @ldata.tbl" >> $CLCOM
fi
if [ -f mldata.tbl ]; then
  echo "imdel @mldata.tbl" >> $CLCOM
fi
echo "logout" >> $CLCOM

(cat $CLCOM | cl) >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT -ne 0 ]
then
 echo "# Error in IRAF ... Exiting #"
 echo "# Check $CLLOG. #" 
 exit 1
fi

if [ -f $GLOBALTMPFILELIST ]
then
  for filename in `cat $GLOBALTMPFILELIST`
  do
    if [ -f $filename ]
    then
      \rm $filename
    fi
  done
fi

if [ $DEBUG -eq 1 ]
then
  if [ -f $TMPFILELIST ]
  then
    for filename in `cat $TMPFILELIST`
    do
      \rm $filename >& /dev/null
    done
  fi
  \rm $TMPFILELIST >& /dev/null
  \rm $GLOBALTMPFILELIST >& /dev/null
fi
